import datetime
import tornado.web
from tornado.escape import xhtml_escape
from ..compat import URLSpec


class TemplateView(tornado.web.RequestHandler):
    # URLSpec name
    name = None
    # HTML template file name
    template_name = None

    @classmethod
    def as_url(cls, pattern):
        return URLSpec(pattern, cls, name=cls.name)

    def make_nav_link(self, name, title):
        active = name == self.name
        return (
            '<li id="head-nav-{name}" class="nav-item{active}">'
            '<a class="nav-link" href="{href}">'
            '{title}{current}'
            '<span id="head-nav-notify-{name}" class="d-none float-right text-warning">'
            ' <span class="fa fa-bell"></span>'
            '</span>'
            '</a>'
            '</li>'
        ).format(
            name=name,
            active=" active" if active else "",
            current=" <span class='sr-only'>(current)</span>" if active else "",
            href=self.reverse_url(name),
            title=xhtml_escape(title),
        )

    def get_context_data(self):
        if self.request.protocol == "http":
            ws_url = "ws://" + self.request.host + \
                    self.application.reverse_url("event_socket")
        else:
            ws_url = "wss://" + self.request.host + \
                    self.application.reverse_url("event_socket")
        return {
            "view_name": self.name,
            "ws_url": ws_url,
            "nav_link": self.make_nav_link,
        }

    def get(self):
        # Do not use the standard render, as we do not need the UIModule
        # machinery
        html = self.render_string(
            self.template_name,
            **self.get_context_data()
            )
        self.finish(html)


class Home(TemplateView):
    name = "home"
    template_name = "machines.html"

    def get_context_data(self):
        _ = self.locale.translate
        ctx = super().get_context_data()
        ctx["title"] = _("Fuss Manager")
        return ctx


class EventMonitor(TemplateView):
    name = "event_monitor"
    template_name = "event_monitor.html"

    def get_context_data(self):
        _ = self.locale.translate
        ctx = super().get_context_data()
        ctx["title"] = _("Event Monitor")
        return ctx


class Playbooks(TemplateView):
    name = "playbooks"
    template_name = "playbooks.html"

    def get_context_data(self):
        _ = self.locale.translate
        playbook_log = self.application.manager.ansible.playbook_log

        ctx = super().get_context_data()
        ctx["title"] = _("Playboog log")
        qmonth = self.get_query_argument("month", None)
        today = datetime.date.today().replace(day=1)
        earliest = playbook_log.get_first_month()
        if qmonth is None:
            month = today
        else:
            month = datetime.datetime.strptime(qmonth, "%Y-%m").date()

        ctx["month"] = month

        if earliest and month > earliest:
            ctx["month_prev"] = (month - datetime.timedelta(days=15)).replace(day=1)
        else:
            ctx["month_prev"] = None

        if month < today:
            ctx["month_next"] = (month + datetime.timedelta(days=40)).replace(day=1)
        else:
            ctx["month_next"] = None

        ctx["log"] = list(playbook_log.list_month(month.year, month.month))
        return ctx
