
class Permission:
    """
    Root class for permissions.

    Permissions do not need to be instantiated: the classes can be used
    directly.
    """
    @classmethod
    def allows(cls, perm):
        """
        Check if this permission is or contains the given permission
        """
        return issubclass(cls, perm)


class ReadonlyBaseAccess(Permission):
    """
    User has read-only access to machine information, user information, and
    events
    """
    pass


class MachineGroupEdit(Permission):
    """
    User can add/remove entries from machine groups
    """
    pass


class ShowPlaybookLogs(Permission):
    """
    User can read playbook logs
    """
    pass


class RunPlaybooks(Permission):
    """
    User can run any playbook
    """
    pass


class InventoryVarsEdit(Permission):
    """
    User can modify host and group variables in the machine inventory
    """
    pass


class MachinePowerOn(Permission):
    """
    User can send power-on wake-on-lan packets
    """
    pass


# TODO: run di playbook (per tag).
