import asyncio
import os
from .base import DataSource, MachineDataSourceMixin
from .. import events, compat
from .inotifywait import Inotifywait


class ArpwatchDataSource(MachineDataSourceMixin, DataSource):
    """
    """
    def __init__(self, config=None, **kw):
        super().__init__(**kw)
        self.fname = config.arpwatch_datafile
        self.watch = Inotifywait(self.fname)
        loop = asyncio.get_event_loop()
        self.started = compat.create_future(loop)

    async def shutdown(self):
        self.watch.stop()

    async def wait_until_started(self):
        await self.started

    async def read_events(self):
        while True:
            event = await self.watch.next_event()
            if event == "MODIFIED":
                await self.parse_file()
            elif event == "END":
                break
            elif event == "START":
                self.started.set_result(True)
                await self.event_hub.publish(
                    events.DataSourceStartEvent(self)
                    )
        await self.event_hub.publish(events.DataSourceStopEvent(self))

    async def run(self):
        await asyncio.gather(
            self.watch.run(),
            self.read_events(),
        )

    @classmethod
    def is_viable(cls, config):
        if not super().is_viable(config):
            return False
        if not os.path.isdir(os.path.dirname(config.arpwatch_datafile)):
            return False
        return Inotifywait.is_viable(config)

    async def parse_file(self):
        with open(self.fname) as fp:
            for l in fp:
                parts = l.split()
                ts = int(parts[2])
                await self.event_hub.publish(events.HostSeenEvent(
                    self, mac=parts[0], ip=parts[1], timestamp=ts,
                ))
